
; Source van dat deel van een diskROM, dat normaal gesproken door de
; fabrikant van een bepaald disk-interface gemaakt wordt
; Bevat voornamelijk de routines voor het eigenlijke lezen/schrijven van
; sectoren, de formatteer-routine etc.
; Voor inpassen van deze code in een diskROM zie na einde van deze source

; Code oorspronkelijk afkomstig uit Philips NMS 8245 machine
; In deze vorm omgezet door Alwin Henseler
; Te assembleren mbv. GEN80 v2.04


; Diverse adressen, systeemvariabelen etc.:


BDOS:	EQU #F37D	; aanroepadres DOS-functies
CMDREG: EQU #7FF8	; commando-register FDC
DATAREG:EQU #7FFB	; data-register FDC
DISINT: EQU #FFCF	; aangeroepen voor langere tijd uitzetten interrupts
DIV16:	EQU #492F	; routine voor uitvoering 16-bits deling
DOSON:	EQU #F368	; schakelt main diskROM in pag.1
DSELREG:EQU #7FFD	; drive-selectie/motor on/in use-register
ENAINT: EQU #FFD4	; aangeroepen na weer aanzetten interrupts
ENASLT: EQU #0024	; schakelt permanent slot in opgegeven pagina
GETSLOT:EQU #402D	; routine voor bepaling slot in pag.1 (=diskROM-slot)
GETWRK: EQU #5FC2	; routine voor opzoeken begin diskROM-werkgebiedje
INTREG: EQU #7FFF	; interrupt/data request bits van FDC
PROMPT: EQU #625A	; routine voor geven melding "Insert diskette for .."
PRVINT: EQU #6027	; routine voor aanroepen vorige interrupt-routine
RAMAD1: EQU #F342	; DOS pag.1 RAM-slot
RAMAD2: EQU #F343	; DOS pag.2 RAM-slot
RAWFLG: EQU #F30D	; <>0: bij elke schrijf-actie data teruglezen
SECBUF: EQU #F34D	; sectorbuffer voor algemene doeleinden
SECTREG:EQU #7FFA	; sector-register FDC
SETINT: EQU #5FF6	; routine voor installeren interrupt-routine op (HL)
SSELREG:EQU #7FFC	; side-selectie register (eigenlijk niet van FDC)
STATREG:EQU #7FF8	; status-register FDC
TRACKREG:EQU #7FF9	; track-register van FDC
XFER:	EQU #F36E	; routine voor LDIR van/naar pag.1-RAM



	ORG #7405	; beginadres 'variabel gebied' in diskROM



; Hier staan een aantal Drive Parameter Blocks, deze worden gebruikt om het
; door een bepaalde drive te bewerken disk-formaat aan te geven of nadere
; gegevens omtrent een bepaald diskformaat te bepalen:

L7405:	DB #F8		; media descriptor
	DW 512		; aantal bytes per sektor
	DB 15		; aantal elementen in 1 direktory-sektor -1
	DB 4		; aantal significante bits in voorgaande gegeven
	DB 1		; cluster masker
	DB 2		; cluster shift
	DW 1		; eerste FAT sektor
	DB 2		; aantal FATs
	DB 112		; aantal elementen in directory
	DW 12		; eerste data sektor
	DW 355		; aantal clusters + 1
	DB 2		; aantal sektoren per FAT
	DW 5		; eerste direktory-sektor

DEFDPB: DB #F9		; media descriptor
	DW 512		; aantal bytes per sektor
	DB 15		; aantal elementen in 1 direktory-sektor -1
	DB 4		; aantal significante bits in voorgaande gegeven
	DB 1		; cluster masker
	DB 2		; cluster shift
	DW 1		; eerste FAT sektor
	DB 2		; aantal FATs
	DB 112		; aantal elementen in directory
	DW 14		; eerste data sektor
	DW 714		; aantal clusters + 1
	DB 3		; aantal sektoren per FAT
	DW 7		; eerste direktory-sektor

; De volgende 2 formaten komen overeen met het standaard 3,5" formaat,
; maar dan met 8 sektoren per track:

	DB #FA		; media descriptor
	DW 512		; aantal bytes per sektor
	DB 15		; aantal elementen in 1 direktory-sektor -1
	DB 4		; aantal significante bits in voorgaande gegeven
	DB 1		; cluster masker
	DB 2		; cluster shift
	DW 1		; eerste FAT sektor
	DB 2		; aantal FATs
	DB 112		; aantal elementen in directory
	DW 10		; eerste data sektor
	DW 316		; aantal clusters + 1
	DB 1		; aantal sektoren per FAT
	DW 3		; eerste direktory-sektor

	DB #FB		; media descriptor
	DW 512		; aantal bytes per sektor
	DB 15		; aantal elementen in 1 direktory-sektor -1
	DB 4		; aantal significante bits in voorgaande gegeven
	DB 1		; cluster masker
	DB 2		; cluster shift
	DW 1		; eerste FAT sektor
	DB 2		; aantal FATs
	DB 112		; aantal elementen in directory
	DW 12		; eerste data sektor
	DW 635		; aantal clusters + 1
	DB 2		; aantal sektoren per FAT
	DW 5		; eerste direktory-sektor

; De volgende routine wordt aangeroepen vanaf adres 4010H aan het begin van
; de diskROM en dient voor het lezen en schrijven van sektoren
; Invoer & uitvoer als voor BIOS-routine PHYDIO (0144H), alleen drive-nummer
; is relatief, dwz. 0 slaat niet op drive A, maar op 1e drive bij deze diskROM

DSKIO:	EI
	JP NC,L75CE

; Hier begint het schrijven van sektoren:

	PUSH AF
	LD A,(RAWFLG)	; verify-vlag
	OR A		; aan of uit ?
	JR L7472	; verder met schrijven sectoren
			; verand. in JR Z,L7472 -> foutcontrole wordt mogelijk

; Hier begint een routine voor het schrijven van sectoren met foutcontrole:

	POP AF
	PUSH AF
	PUSH BC
	PUSH DE
	PUSH HL
	CALL L7498	; schrijf sectoren (excl.foutcontrole)
	POP HL
	POP DE
	POP BC
	JR C,L746E	; breek af in geval van schrijffout

; Hier begint de controle van geschreven sectoren:

	CALL L782C	; reset FDC
	POP AF
	CALL L7566	; verifieer geschreven sectoren
	JR L7476
L746E:	POP IY
	JR L7476

; Hier staat een routine voor het schrijven van sectoren zonder foutcontrole:

L7472:	POP AF
	CALL L7498	; schrijf sectoren

; Hier worden diskacties beeindigd:

L7476:	EI
	CALL ENAINT	; aanroep hook na aanzetten interrupts
	PUSH AF
	LD C,50 	; waarde voor tijd dat uitgezette drive nog doorloopt
	JR NC,L7481
	LD C,0
L7481:	CALL L782C	; reset FDC
	LD (IX+0),150	; waarde voor tijd dat drive-lampje aan blijft (3 sec)
	LD A,(IX+3)	; actieve fysische drive
	AND A
	JR NZ,L7493
	LD (IX+1),C	; ready-toestand eerste drive
	POP AF
	RET
L7493:	LD (IX+2),C	; ready-toestand tweede drive
	POP AF
	RET

; Hier begint het schrijven van sectoren:

L7498:	CALL L76D6	; selecteren drive & instelling kop-positie
	RET C		; breek af in geval van ongeldige media descriptor
	CALL DISINT
	DI
L74A0:	BIT 7,H 	; schrijven uit pag.2/3 ?
	JR NZ,L74CD	; zo ja, direkt gaan schrijven

; In geval van schrijven van sectoren uit pag.0 of 1 gebeurt dit via een
; aantal hulproutines die verplaatst worden naar een sectorbuffer op (F34DH):

	CALL L7D92	; relocaten verscheidene routines
	CALL L7D2D	; schrijven sectoren via pag.3-hulproutine(s)
	RET C		; breek af in geval van diskfout
	INC B
	DEC B		; nog meer sectoren te schrijven ?
	RET Z		; zo nee, klaar
	CALL DISINT
	DI
	BIT 7,H 	; controle of volg.sector pag.1/2 grens overlapt
	JR NZ,L74CD	; zo nee, verder met normale manier van schrijven

; Indien een te schrijven sector precies over de grens van pag.1 en 2 loopt,
; wordt de volgende procedure gehanteerd:

	PUSH HL
	PUSH DE
	PUSH BC
	LD DE,(SECBUF)	; adres sectorbuffer
	PUSH DE
	LD BC,512
	CALL XFER	; uitvoering LDIR vanuit standaard DOS RAM-slot
	POP HL
	POP BC
	POP DE
	CALL L74D8	; schrijf 1 sektor uit pag.3-buffer
	POP HL
	JR L74D0

; Hier begint het schrijven van sektoren die zich in pag.2 of 3 bevinden:

L74CD:	CALL L74D8	; schrijf 1 sektor naar disk
L74D0:	RET C		; breek af in geval van diskfout
	DEC B		; nog meer sectoren te schrijven ?
	RET Z		; zo nee, klaar
	CALL L77A1	; stel FDC in op volgende sector
	JR L74A0	; ga verder met sectoren schrijven

; Routine voor het schrijven van 1 sektor:

L74D8:	LD E,#0B	; bepaalt manier van reageren op CRC/seek errors ed.
L74DA:	CALL L77F4	; wacht tot FDC klaar is voor volgende opdracht
	LD A,#A0	; FDC-commando 'write sector'
	BIT 6,D 	; SS of DS disktype ?
	JR Z,L74EB	; zo nee, geen diskzijde aan FDC opgeven
	OR %00000010	; zet controle op diskzijde in FDC-commando
	BIT 0,D 	; diskzijde
	JR Z,L74EB
	OR %00001000	; verwerk diskzijde in FDC-commando
L74EB:	PUSH HL
	PUSH DE
	PUSH BC
L74EE:	LD DE,L7504	; beginadres routine voor beeindiging schrijven sektor
	PUSH DE
L74F2:	LD BC,INTREG	; adres FDC interrupt/data request register
L74F5:	LD (CMDREG),A	; geef FDC opdracht voor schrijven sektor
L74F8:	LD A,(BC)
	ADD A,A
	RET P		; interrupt-request -> einde sektor of diskfout
	JR C,L74F8	; blijf wachten op data request
	LD A,(HL)	; haal byte op uit geheugen
L74FE:	LD (DATAREG),A	; geef byte door aan FDC
	INC HL		; verhoog adrespointer
	JR L74F8	; wacht op vraag FDC naar volgende byte

; Hier eindigt het schrijven van een sektor:

L7504:	POP BC
	POP DE
	POP HL
L7507:	LD A,(STATREG)	; vraag resultaat op van FDC
	AND %11111100
	JR NZ,L751B
	BIT 7,H 	; staat volgende sector in pag.2 of 3 ?
	JR NZ,L7563
	DEC B		; verlaag aantal nog te lezen sectoren
	JR Z,L7563
L7515:	CALL L77A1	; stel FDC in op volgende sector
L7518:	JP 0		; vervolgadres wordt ingevuld bij relocaten
L751B:	BIT 6,A 	; write protect fout opgetreden ?
	JR NZ,L7545
	PUSH AF
L7520:	CALL L77DB	; eventueel herplaatsen kop op zelfde spoor
	POP AF
	DEC E		; aantal pogingen
	JR NZ,L74DA
	SCF		; niet gelukt -> diskfout

; Hier wordt bij een schrijffout adhv. de FDC statusbits een passende
; foutcode bepaald:

	LD E,A
	BIT 7,E 	; not ready ?
	LD A,2		; foutcode: 'not ready'
	JR NZ,L7563
L752F:	BIT 5,E 	; zou altijd 0 moeten zijn bij schrijven
	LD A,10 	; foutcode: 'verify error'
	JR NZ,L7563
	BIT 4,E 	; record not found ?
	LD A,8		; foucode: 'record not found'
	JR NZ,L7563
	BIT 3,E 	; CRC error ?
	LD A,4		; foutcode: 'CRC error'
	JR NZ,L7563
	LD A,12 	; foutcode: 'other error'
	JR L7563

; In het geval van een 'write protect' melding van de FDC, wordt hier bepaald
; of er werkelijk sprake is van een schrijfbeveiligde disk of dat er gewoon
; helemaal geen disk aanwezig is:

L7545:	LD A,#80	; FDC-commando 'read sector'
L7547:	LD (CMDREG),A	; probeer gezochte sector te lezen
L754A:	LD HL,INTREG
	LD DE,0 	; bepaalt max.wachttijd op respons (ca. 1.2 sec)
L7550:	LD A,(HL)
	RLCA		; data request van FDC ?
	JR C,L7561	; zo ja, schrijfbeveiligde disk aanwezig
	RLCA		; interrupt request van FDC ?
	JR NC,L7561	; zo ja, idem
	DEC DE
	LD A,E
	OR D
	JR NZ,L7550	; blijf wachten op reactie van FDC
	LD A,2		; foutcode: 'not ready'
	SCF
	JR L7563
L7561:	XOR A		; foutcode: 'write protected'
	SCF
L7563:	RET
	DW 0		; RET+DW worden bij relocaten vervangen door jump

; Hier begint een routine voor het verifieren van een aantal sektoren:

L7566:	CALL L76D6	; instelling drive-selectie en kop op juiste spoor
	RET C
	CALL DISINT	; aanroep hook voor langere tijd uitzetten interrupts
	DI
L756E:	BIT 7,H 	; te verifieren sektor in pag.2/3 ?
	JR NZ,L75C1

; Als de te verifieren sektoren niet in pag.2 of 3 liggen, wordt de FDC-
; aansturing gedaan via een aantal hulproutines die naar een sectorbuffer
; gekopieerd worden:

	CALL L7DAA	; installatie hulproutines in sectorbuffer
	LD E,#80	; soortement vlag mbt. reaktie op CRC/seek errors ed.
	CALL L7D2D	; verificatie sectoren mbv. hulproutines in pag.3
	RET C		; breek af in geval van diskfout
	INC B
	DEC B		; nog meer sektoren te controleren ?
	RET Z		; zo nee, beeindig disk-access
	CALL DISINT
	DI
	BIT 7,H 	; ligt beginadres volgende sector in pag.2 of 3 ?
	JR NZ,L75C1	; zo ja, verifieer volgende sectoren rechtstreeks

; Als een te controleren sector de grens tussen pag.1 en 2 overlapt, wordt
; de volgende procedure gehanteerd:

	PUSH HL
	PUSH DE
	PUSH BC
	LD DE,(SECBUF)	; beginadres pag.3-sectorbuffer
	DEC D
	DEC D
	LD BC,512
	CALL XFER	; te controleren sector naar andere sectorbuffer
	POP BC
	POP DE
	PUSH DE
	PUSH BC
	LD HL,(SECBUF)
	LD E,#0B	; bepaald gedrag bij leesfouten
	CALL L761F	; lees sector in in pag.3-sectorbuffer
	JR C,L75BB	; breek af in geval van diskfout
	LD HL,(SECBUF)
	LD D,H
	LD E,L
	DEC D
	DEC D		; DE=adres andere sectorbuffer
	LD BC,512
L75AD:	LD A,(DE)	; byte uit geheugen
	CPI		; vergelijk met teruggelezen byte
	JR NZ,L75B8	; verschil ? -> eindig met 'verify error'
	INC DE
	JP PE,L75AD	; vergelijk volgende byte
	JR L75BB
L75B8:	LD A,10 	; foutcode: 'verify error'
	SCF
L75BB:	POP BC
	POP DE
	POP HL
	RET C		; breek af in geval van diskfout
	JR L75C7	; ga door met controle evt.volgende sectoren

; Hier begint het verifieren van sectoren die geheel in pag.2 of 3 liggen:

L75C1:	LD E,#80	; bepaalt gedraag in geval van CRC/seek errors ed.
	CALL L761F	; verifieer sektor
	RET C		; breek af in geval van disk/verificatie-fout
L75C7:	DEC B		; nog meer sectoren te controleren ?
	RET Z		; zo nee, beeindig disk-access
	CALL L77A1	; stel FDC in op volgende sector
	JR L756E	; vervolg met controleren sectoren

; Hier begint het lezen van sektoren:

L75CE:	CALL L75D4	; uitvoering sektoren lezen
	JP L7476	; eindig met vastleggen ready-toestand drive A/B

L75D4:	CALL L76D6	; activeer juiste drive & zet kop op juiste spoor
	RET C		; keer terug indien media descriptor onbekend is
L75D8:	CALL DISINT	; aanroep hook voor uitzetten interrupts
	DI
L75DC:	BIT 7,H 	; disk-I/O naar pag.2/3 ?
	JR NZ,L7612

; Hier begint de installatie van een hulproutine voor disk-I/O van/naar
; pagina 0 of 1 (altijd van/naar standaard RAM-slot):

	CALL L7DCD	; verplaatst sektor-leesroutines naar FAT-sectorbuffer
	LD E,#0B	; bepaalt manier van reageren bij CRC/seek error oid.
	CALL L7D2D	; aanroep sektor-leesroutines in pag.3
	RET C		; eindig indien leesfout
	INC B
	DEC B		; alle sektoren gelezen ?
	RET Z
	CALL DISINT
	DI
	BIT 7,H 	; disk-I/O naar pag.2/3 ?
	JR NZ,L7612

; Bij disk-I/O naar een adres waar het bestemmingsadres de grens tussen
; pag.2 en pag.3 overlapt, wordt nog een 3e werkwijze gehanteerd:

	PUSH HL 	; bewaar 'eigenlijk DTA-adres'
	LD HL,(SECBUF)
	LD E,#0B
	CALL L761F	; lees sektor in in pag.3-buffer
	POP HL
	RET C		; breek af in geval van diskfout
	PUSH HL
	PUSH DE
	PUSH BC
	EX DE,HL
	LD HL,(SECBUF)
	LD BC,512
	CALL XFER	; voor LDIR van/naar pag.1-RAM uit
	POP BC
	POP DE
	POP HL
	AND A
	JR L7618

; Hier wordt naar toe gesprongen bij het lezen van sektoren naar pag.2/3:

L7612:	LD E,#0B	; bepaald manier van reageren bij seek error oid.
	CALL L761F	; lezen v.e. sektor
	RET C		; afbreken bij diskfout
L7618:	DEC B		; nog meer sektoren te lezen ?
	RET Z		; zo nee, klaar
	CALL L77A1	; ophogen sektornr./evt.opschuiven drive-kop
	JR L75DC	; volgende sektor lezen

; Hier begint het eigenlijke lezen v.e. sektor:

L761F:	CALL L77F4	; wacht tot FDC klaar is voor een opdracht
	LD A,#80	; FDC-commando 'read sector'
	BIT 6,D 	; SS of DS disktype ?
	JR Z,L7630	; SS -> diskzijde weglaten uit FDC-commando
	OR 2		; zet controle op diskzijde in FDC-commando
	BIT 0,D 	; diskzijde
	JR Z,L7630
	OR 8		; geef diskzijde 1 aan in FDC-commando
L7630:	PUSH HL
	PUSH DE
	PUSH BC
L7633:	LD BC,L766E	; routine voor beeindiging lezen sektor
	PUSH BC
L7637:	LD BC,INTREG	; adres interrupt/data request register FDC
	BIT 7,E 	; lezen of verifieren ?
L763C:	JR NZ,L76A6
L763E:	LD (CMDREG),A	; geef FDC-commando 'read sector'
	LD DE,23808	; voor max.wachtijd ca. 1/3 sec. bij zoeken sector
L7644:	LD A,(BC)	; interrupt/data-request FDC
	ADD A,A
	JR NC,L7663	; data request -> eerste byte lezen
	RET P		; interrupt request -> einde sektor of leesfout
	DEC E
	JR NZ,L7644
	LD A,(BC)
	ADD A,A
	JR NC,L7663
	RET P
	DEC D
	JR NZ,L7644
	POP BC
	POP BC
	POP DE
	POP HL
	LD A,2		; foutcode: 'not ready'
	SCF
L765B:	JP L76A3	; klaar

; wacht op volgende byte van FDC:

L765E:	LD A,(BC)
	ADD A,A
	RET P
	JR C,L765E

; Haal een byte op van de floppycontroller:

L7663:	LD A,(DATAREG)

; In gerelocate toestand wordt bij verifieren over het volgende stukje code
; een ander stukje (76CE-76D5) gekopieerd, die niet de bytes in het geheugen
; zet maar alleen vergelijkt met de geheugeninhoud:

L7666:	LD (HL),A	; zet in geheugen
	INC HL		; verhoog adres
	JR L765E
	DS 4,0		; verschil byte-vergelijk / byte-leesroutines

; Hier eindigt het lezen v.e. sektor:

L766E:	POP BC
	POP DE
	POP HL
L7671:	LD A,(STATREG)
	AND %10011100	; leesfout opgetreden ?
	JR NZ,L7685	; zo ja, bepaal bijbehorende foutcode
	BIT 7,H 	; disk-I/O naar pag.2/3 ?
	JR NZ,L76A3	; zo ja, eindig met return
	DEC B		; verlaag aantal te lezen sectoren
	JR Z,L76A3
L767F:	CALL L77A1	; ophogen sect.nr./evt.verder zetten drive-kop
L7682:	JP 0		; adres wordt ingevuld bij relocaten

; Hier wordt naar toe gesprongen wanneer het FDC-statusregister een fout
; aangeeft:

L7685:	PUSH AF
L7686:	CALL L77DB	; evt. kop opnieuw op juiste spoor zetten
	POP AF
	DEC E		; aantal pogingen om sektor te lezen
	JR NZ,L761F

; Hier wordt adhv. de FDC-statusbits een passende foutcode bepaald:

	SCF
	LD E,A
	BIT 7,E 	; toestand ready-ingang FDC
	LD A,2		; foutcode: 'not ready'
	JR NZ,L76A3
	BIT 4,E 	; record not found ?
	LD A,8		; foutcode: 'record not found'
	JR NZ,L76A3
	BIT 3,E 	; CRC error ?
	LD A,4		; foutcode: 'CRC error'
	JR NZ,L76A3
	LD A,12 	; foutcode: 'other error' (bv. lost data)
L76A3:	RET
	DW 0		; RET + DW worden bij relocaten vervangen door jump

; Hier begint het verifieren van een sektor:

L76A6:	LD (CMDREG),A	; geef FDC-commando 'read sector'
	LD DE,23808	; waarde voor max.wachttijd bij zoeken sector (1/3 s)
L76AC:	LD A,(BC)	; toestand FDC interrupt/datarequest
	ADD A,A
	JR NC,L76CB	; data request -> byte van FDC ophalen
	RET P		; interrupt request -> einde sektor of leesfout
	DEC E
	JR NZ,L76AC	; blijf wachten
	LD A,(BC)
	ADD A,A
	JR NC,L76CB
	RET P
	DEC D
	JR NZ,L76AC
	POP BC
	POP BC
	POP DE
	POP HL
L76C0:	LD A,2		; foutcode: 'not ready'
	SCF
	JP L76A3	; beeindig disk-access
L76C6:	LD A,(BC)	; toestand FDC interrupt/data request
	ADD A,A
	RET P		; interrupt request -> einde sektor of leesfout
	JR C,L76C6
L76CB:	LD A,(DATAREG)	; haal byte op van FDC

; Bij het verifieren van sectoren via hulproutines in pag.3 wordt de
; sector-leesroutine aangepast met het volgende stukje code:

L76CE:	CP (HL) 	; gelijk aan byte uit geheugen ?
	INC HL
	JR Z,L76C6	; zo ja, vervolg met wachten op volgende byte
	LD A,10 	; foutcode: 'verify error'
	JR L76C0

; Deze routine zorgt voor het selecteren van een bepaalde drive en het
; positioneren van de drive-kop op het juiste spoor:

L76D6:	PUSH AF 	; rel.drivenr.
	PUSH BC 	; aant.sekt/media descr.
	PUSH HL 	; DTA-adres
	CALL GETWRK	; bepaal adres 8-byte werkgebiedje
	POP HL
	POP BC
	POP AF
	CP 2		; rel.drivenr. > 2 ?
	JR C,L76E7	; zo nee, doorgaan met uitzoeken disk-type
L76E3:	LD A,12 	; foutcode: 'other error'
	SCF
	RET

; Hier wordt het disk-type uitgezocht adhv. de media descriptor:

L76E7:	PUSH AF
	LD A,C
	CP #F8		; med.descr.kleiner F8H ?
	JR C,L76F1	; zo ja, eindig met 'other error'
	CP #FC		; med.descr.> FBH ?
	JR C,L76F4	; zo nee, ga door
L76F1:	POP AF
	JR L76E3	; eindig met 'other error'
L76F4:	EX (SP),HL
	PUSH HL
	PUSH BC 	; media descr.
	CALL L77F4	; wacht tot FDC klaar is voor een opdracht
	BIT 1,C 	; type met 8 of 9 sektoren / track ?
	LD C,E
	LD B,D
	LD DE,8
	JR NZ,L7704
	INC E
L7704:	CALL DIV16	; BC=BC/DE, rest in HL
	LD A,L
	INC A		; rel.sektoren 0-8 hebben voor de FDC nrs. 1-9
	LD (SECTREG),A	; zet rel.sectornr. in FDC-register
	LD L,C		; sektornr./sectoren per track
	POP BC		; aantal sectoren, med.descr.
	POP AF		; rel.drivenr.
	LD H,A
	XOR A
	BIT 0,C 	; SS of DS ?
	JR Z,L771A
	SRL L		; bereken tracknummer
	JR NC,L771A	; sector op zijde 0 ?
	INC A		; op zijde 1
L771A:	LD (SSELREG),A	; instelling side-select  register
	LD D,A
	LD A,(IX+7)	; aantal hardware drives
	DEC A
	JR Z,L7725	; 1 drive -> altijd hardware-drive A selecteren
	LD A,H		; opnemen rel.drivenr. in drive-sel.register
L7725:	OR %11000100	; waarde voor motor/in use & drive aan
	PUSH AF
	LD A,C		; media descr.
	RRCA
	RRCA
	AND %11000000
	OR D
	LD D,A		; bit 7: 0=9, 1=8 sectoren/track
			; bit 6: 0=SS, 1=DS
			; bit 0=diskzijde (altijd 0 voor SS)
	LD C,L		; tracknr.
	LD A,(IX+7)
	DEC A		; 1 of 2 hardware drives ?
	JR Z,L7759	; 1 drive -> bewaren kopposities in geheugen overbodig
	LD A,(IX+3)	; actieve fysische drive
	CP H		; zelfde als nu aangesproken wordt ?
	JR Z,L7770	; zo ja, geen kop-posities omwisselen

; Hier wordt van drive gewisseld, waarbij de kopstand van de andere drive
; van de FDC gehaald en bewaard wordt in het geheugen:

	XOR 1		; wisseling eerste/tweede drive
	LD (IX+3),A	; vastleggen nu actieve fysische drive
	LD A,(TRACKREG) ; aan FDC bekende kop-positie
	JR Z,L774E

; wisseling eerste -> tweede drive:

	LD (IX+4),A	; leg kop-positie eerste drive in geheugen vast
	LD A,(IX+5)	; haal kop-positie tweede drive uit geheugen
	JR L7754

; wisseling tweede -> eerste drive:

L774E:	LD (IX+5),A	; leg kop-positie tweede drive vast in geheugen
	LD A,(IX+4)	; haal kop-positie eerste drive uit geheugen
L7754:	LD (TRACKREG),A ; geef kop-positie actieve drive door aan FDC
	JR L7770	; ga verder met aanzetten drive

; Hier begint een eventuele drive-wisseling in het geval van 1 hardware-drive:

L7759:	LD A,(IX+6)	; actieve logische drive
	CP H		; zelfde als nu aangesproken wordt ?
	LD (IX+6),H	; leg nu actieve logische drive vast
	JR Z,L7770	; zelfde drive -> geen disk-wissel melding
	PUSH IX
	PUSH DE
	PUSH BC
	CALL MTOFF	; uitzetten drives
	CALL PROMPT	; geef diskwissel-melding
	POP BC
	POP DE
	POP IX
L7770:	POP AF
	PUSH BC
	PUSH AF
	DI
	LD A,(IX+0)
	LD B,A
	POP AF
	LD (DSELREG),A	; zet drive aan
	LD (IX+0),#FF	; zet drive-uit tellertje onbeperkt aan
	EI
	LD A,(TRACKREG)
	CP C		; drive-kop op juiste spoor ?
	CALL NZ,L77E1	; zo nee, zet hem daar op
	LD A,B		; 'toestand drive-lampje'
	AND A		; stond drive nog aan ?
	JR NZ,L7793	; zo ja, aanzetten drive-motor overslaan

; Als de drive(s) niet nog aan stond(en) sinds een vorige diskactie, wordt
; hier 1 seconde gewacht zodat de motor van de drive(s) op toeren kunnen
; komen:

	LD B,100
L778E:	CALL L7796	; wacht ca. 10 ms
	DJNZ L778E	; 100 keer = 1 sec.
L7793:	POP BC
	POP HL
	RET
L7796:	PUSH HL
	LD HL,1190	; voor vertraging ca. 10 ms
L779A:	DEC HL
	LD A,H
	OR L
	JR NZ,L779A
	POP HL
	RET

; Deze routine zorgt voor het ophogen van het sektornummer en eventueel
; verder zetten van de drive-kop:

L77A1:	CALL L77F4	; wacht tot FDC klaar is voor een opdracht
	INC H
	INC H		; verhoog DTA-adres
L77A6:	LD A,(SECTREG)
	INC A
L77AA:	LD (SECTREG),A	; verhoog rel.sect.nr.
	BIT 7,D 	; 8 of 9 sectoren/track ?
	JR NZ,L77B4
	CP 10
	RET C		; sectornr. 1-9 -> nog op zelfde track & diskzijde
L77B4:	CP 9		; sectornr. 1-8 ?
	RET C		; zo ja, geen verandering track/diskzijde
	LD A,1
L77B9:	LD (SECTREG),A	; reset relatief sect.nr.
	BIT 6,D 	; SS of DS ?
	JR Z,L77CB	; SS -> direkt tracknr. ophogen
	BIT 0,D 	; diskzijde 0 of 1 ?
	JR NZ,L77CB	; diskzijde 1 -> verder op volgend spoor
	SET 0,D
L77C6:	LD (SSELREG),A	; kies diskzijde 1
	JR L77DA	; klaar
L77CB:	RES 0,D
	XOR A
L77CE:	LD (SSELREG),A	; diskzijde = 0
	INC C		; verhoog tracknr.
L77D2:	CALL L77F4	; wacht tot FDC klaar is voor een opdracht
	LD A,#50	; FDC-commando 'step in'
L77D7:	LD (CMDREG),A	; schuif drive-kop 1 spoor verder
L77DA:	RET

; Hier wordt naar toe gesprongen bij diskfouten als CRC-error, seek error etc:

L77DB:	BIT 0,E 	; geeft aan af er 'gezaagd' gaat worden
	RET NZ
L77DE:	CALL L77FB	; 0 -> kop terugschuiven en opnieuw op spoor zetten

; De volgende routine zet de drive-kop op track C:

L77E1:	CALL L77F4
	LD A,C
	OR A		; gewenste spoor = 0 ?
	JR Z,L7802	; zo ja, kop plaatsen dmv. 'restore'
L77E8:	LD (DATAREG),A	; geef gewenste track aan FDC door dmv. datareg.(!)
	EX (SP),HL
	EX (SP),HL	; wacht heel even
	LD A,#10
L77EF:	LD (CMDREG),A	; geef FDC 'seek track' commando
	JR L780A	; voeg settling time in

; De volgende routine wacht tot de FDC klaar is met een opdracht:

L77F4:	LD A,(STATREG)
	RRA		; busy-bitje van FDC -> carry
	JR C,L77F4	; blijf wachten zolang FDC bezig is
	RET

L77FB:	LD A,(TRACKREG)
	OR A		; kop op spoor 0 ?
L77FF:	CALL Z,L7815	; zo ja, zet hem eerst even op spoor 4

L7802:	CALL L77F4	; wacht tot FDC klaar is
	LD A,0
L7807:	LD (CMDREG),A	; geef FDC 'restore' opdracht

; Hier volgt een zgn. 'settling time', bedoeld om de lees- / schrijfkop
; stabiel op het disk-oppervlak te laten komen:

L780A:	PUSH HL
	LD HL,2499	; voor wachttijd van ca. 21 ms.
L780E:	DEC HL
	LD A,H
	OR L
	JR NZ,L780E
	POP HL
	RET

L7815:	PUSH BC
	LD C,4
L7818:	CALL L77E1	; zet drive-kop op spoor 4
	POP BC
	RET

INIHRD: CALL L782C	; afbreken evt. FDC-commando
	JR L7826

; De volgende routine wordt aangeroepen vanaf adres 401FH aan het begin van
; de diskROM en zorgt voor het uitzetten van de bij deze diskROM horende
; drives:

MTOFF:	LD (IX+0),0	; reset drive-uit tellertje

L7826:	LD A,%00000011
	LD (DSELREG),A	; zet alle drive-select signalen op non-actief
	RET

L782C:	LD A,#D0
	LD (CMDREG),A	; FDC 'force interrupt' commando
	EX (SP),HL
	EX (SP),HL
	EX (SP),HL
	EX (SP),HL	; even wachten voor FDC
	LD A,(DATAREG)
	LD A,(STATREG)	; uitlezen tbv. resetten FDC-statusbits
	RET

; Hier wordt bepaald hoeveel drives aangestuurd worden:

DRIVES: PUSH BC 	; C bevat aantal al geinstalleerde drives
	PUSH AF 	; Z-vlag gezet -> opgestart met CTRL
	CALL GETWRK	; zoek beginadres 8-byte werkgebiedje op
	POP AF
	PUSH AF
	JR Z,L785C	; CTRL ingedrukt -> 1 drive aansturen
	LD A,%11000101
	LD (DSELREG),A	; zet drive B aan
	CALL L77FB	; geef 'restore' commando aan FDC
	LD HL,30410	; voor max. wachttijd van ca. 0.5 sec
L7850:	LD A,(STATREG)
	AND %00000100	; kop v. drive B op spoor 0 geland ?
	JR NZ,L7860	; zo ja -> drive B aanwezig
	DEC HL
	LD A,H
	OR L
	JR NZ,L7850
L785C:	LD L,1		; 1 drive aansturen
	JR L7865
L7860:	LD L,2
	CALL L787A
L7865:	LD A,%11000100
	LD (DSELREG),A	; selecteer drive A
	CALL L787A
	LD (IX+7),L	; leg aantal aangestuurde drives vast
	CALL L7826	; zet drive-selectiesignalen uit
	POP AF
	JR Z,L7878
	LD L,2
L7878:	POP BC
	RET

L787A:	PUSH DE
	RES 5,D
	CALL L77FB
	CALL L77FB
	CALL L77FB	; heen- en weer zetten drive-kop tussen spoor 0 en 4
	POP DE
	RET


; Hier volgt de initialisatie van een 8-byte werkgebiedje en een
; interrupt-routine:

INIENV: CALL GETWRK	; bepaal beginadres 8-byte werkgebiedje
	LD B,7
	XOR A
L788E:	LD (HL),A
	INC HL
	DJNZ L788E
	LD HL,L7898	; beginadres interruptroutine van deze diskROM
	JP SETINT	; vervolg met installatie interrupt-routine

; Hier volgt de interrupt-routine voor deze diskROM:

L7898:	PUSH AF
	CALL GETWRK	; bepaal begin 8-byte werkgebiedje
	LD A,(HL)	; 'drive-uit tellertje'
	AND A		; op 0 gekomen ?
	JR Z,L78A8	; zo ja, drive(s) uitzetten
	CP #FF		; moet drive continu aan blijven ?
	JR Z,L78AB	; zo ja, ook niks met drive-uit teller(s) doen
	DEC A		; verlaag tellertje
	LD (HL),A
	JR NZ,L78AB	; indien nog niet op 0, drives niet uitzetten
L78A8:	CALL L7826	; zet alle drive-selectiesignalen op non-actief
L78AB:	INC HL
	LD A,(HL)	; indicatie ready-toestand drive A
	AND A
	JR Z,L78B1
	DEC (HL)	; verlaag teller die aangeeft hoelang drive al uit is
L78B1:	INC HL
	LD A,(HL)	; indicatie ready-toestand drive B
	AND A
	JR Z,L78B7
	DEC (HL)
L78B7:	POP AF
	JP PRVINT	; vervolg met int.routines andere diskROMs

; De volgende routine wordt aangeroepen van adres 4013H aan het begin van
; de diskROM en dient voor het opvragen van de 'disk change status' van een
; drive
; Invoer: A=drive nummer (relatief)
;	  B=0
;	  C=media descriptor
;	  HL=beginadres DPB
; Uitvoer: C-vlag geeft aan of een disk-leesfout is opgetreden
;	   Indien geen fout (carry=0) dan bevat B de disk-change status:
;	0: niet zeker of disk gewisseld is
;	1: disk niet gewisseld
;	255: disk gewisseld

DSKCHG: EI		; waarschijnlijk ivm. teruglopen drive-uit teller(s)
	PUSH HL 	; adres DPB
	PUSH BC 	; media descriptor
	PUSH AF 	; relatief drive-nummer
	CALL GETWRK	; zet beginadres 8-byte werkgebiedje in IX en HL
	POP AF
	POP BC
	POP HL
	AND A		; eerste of 2e drive ?
	LD B,(IX+2)	; ready-status tweede drive
	JR NZ,L78CE
	LD B,(IX+1)	; ready-status eerste drive
L78CE:	INC B
	DEC B		; loopt betreffende drive nog ?
	LD B,1
	RET NZ		; zo ja, mag aangenomen worden dat zelfde disk nog zit
	PUSH BC
	PUSH HL
	LD DE,1
	LD HL,(SECBUF)
	PUSH AF
	CALL DSKIO	; lees 1e FAT sektor
	JR C,L78F9	; in geval van leesfout (of lege drive)
	POP AF
	LD HL,(SECBUF)
	LD B,(HL)	; nieuw gelezen media descriptor
	POP HL
	PUSH BC
	CALL GETDPB	; ophalen bijbehorend DPB
	LD A,10
	JR C,L78FA	; med.descr. onbekend (-> 'verify error' ??)
	POP AF		; nieuw gelezen media descriptor
	POP BC		; 'oude' med. descr.
	CP C		; nog zelfde soort disk in de drive ?
	SCF
	CCF
	LD B,255
	RET NZ		; ander type disk -> disk is gewisseld
	INC B		; nog zelfde soort disk -> onbekend of gewisseld is
	RET
L78F9:	POP DE
L78FA:	POP DE
	POP DE
	RET

; De volgende routine wordt oa. aangeroepen vanaf adres 4016H aan het begin
; van de diskROM en zorgt voor het ophalen van een Drive Parameter Block
; adhv. een gegeven 'media descriptor'
; Invoer: B=media descriptor
;	  HL=adres waarop bijbehorend DPB moet komen
; Uitvoer: C-vlag gezet: media type niet ondersteund
;	   anders: DPB ingevuld

GETDPB: EX DE,HL
	INC DE		; eerste byte DPB (drive nummer) overslaan
	LD A,B
	SUB #F8
	RET C		; type 0-F7H: niet gedefinieerd binnen MSX-systeem
	CP #FA+1-#F8	; media descriptor=F8,F9,FA of FBH ?
	JR NC,L791B	; nee: dit type niet ondersteund door deze diskROM
	RLCA
	LD C,A
	LD B,0
	LD L,A
	LD H,B
	ADD HL,HL
	ADD HL,HL
	ADD HL,HL
	ADD HL,BC
	LD BC,L7405	; adres DPB's van ondersteunde media-types
	ADD HL,BC
	LD BC,18
	LDIR		; kopieer juiste DPB naar bestemmingsadres
	RET
L791B:	LD A,10 	; foutcode
	SCF		; geeft aan: niet ondersteund (wel bekend) type
	RET

	DS 18,0 	; naar alle waarschijnlijkheid niet gebruikt

; Hier staat een string met daarin de formaat-keuze voor formatteren
; Dit adres kan opgevraagd worden via de entry op adres 4019H in de diskROM:

L7931:	DB 13,10
	DB "1 - Single sided, 80 tracks",13,10
	DB "2 - Double sided, 80 tracks",13,10,0

; De volgende routine wordt aangeroepen vanaf adres 4019H aan het begin van
; de diskROM en geeft in HL het adres van voorgaande string
; (HL=0 betekent: geen keuze mogelijk)

CHOICE: LD HL,L7931
	RET

	DS 16,0 	; naar alle waarschijnlijkheid niet gebruikt

; De volgende routine wordt aangeroepen vanaf adres 401CH aan het begin van
; de diskROM en behelst de eigenlijke formatteer-routine:
; Invoer: A=formatteer-keuze 1-9 (SS/DD)
;	  D=drive nummer 0-... (relatief)
;	  HL=beginadres buffer voor formatteer-routine
;	  BC=lengte van deze buffer
; Uitvoer: C-vlag=0: alles o.k.
;	   C-vlag=1 -> A=foutcode

DSKFMT: PUSH HL
	LD HL,-7000	; minimaal benodigde ruimte voor formatteer-buffer
	ADD HL,BC
	POP HL
	JR C,L799D	; genoeg -> begin formatteerproces
	SCF
	LD A,14 	; foutcode: 'not enough memory'
	RET

L798E:	POP HL
L798F:	SCF
	LD A,12
	RET

L7993:	POP DE
	POP BC
	POP HL
	JP L7476	; eindig met vastleggen ready-toestand drive(s)

L7999:	POP BC
	JP L7476

; Voor het formateren wordt als eerste SS/DS bepaald adhv.de meegegeven keuze:

L799D:	PUSH AF
	PUSH HL
	CALL GETWRK	; stel beginadres 8-byte werkgebiedje vast
	POP HL
	POP AF
	OR A		; keuze = 0 ?
	JR Z,L798F	; zo ja, eindig met 'Bad parameter'
	CP 3		; keuze > 2 ?
	JR NC,L798F	; zo ja, idem
	DEC A		; SS of DS ?
	JR Z,L79BF
	LD BC,#50F9	; 80 tracks DS formatteren
	JR L79C2

	DS 12,0 	; waarschijnlijk niet gebruikt

L79BF:	LD BC,#50F8	; 80 tracks SS formatteren
L79C2:	PUSH HL
	BIT 0,C 	; SS ?
	JR Z,L7A09	; zo ja, controle op side-selectie overslaan

; Bij DS formatteren wordt als eerste het juiste functioneren van de
; side selectie gecontroleerd:

	PUSH BC
	PUSH DE
	LD A,D
	LD DE,0
	PUSH BC
	CALL L76D6	; stel FDC/drive in op spoor 0, diskzijde 0
	POP BC
	RES 7,C
	CALL L7A4E	; opbouw track-invulling in formatteer-buffer
	CALL L7BD8	; schrijf track op diskzijde 0
	JR C,L7993	; afbreken in geval van diskfout
	POP DE
	PUSH DE
	LD A,D
	LD DE,9
	PUSH BC
	CALL L76D6	; stel FDC/drive in op spoor 0, diskzijde 1
	POP BC
	SET 7,C
	CALL L7A4E	; opbouw trackinhoud in buffer
	CALL L7BD8	; schrijven track op diskzijde 1
	JR C,L7993	; afbreken igv. diskfout
	CALL L782C	; reset FDC
	LD HL,(SECBUF)
	LD BC,#01F9
	POP DE
	LD A,D
	PUSH DE
	LD DE,0
	OR A
	CALL DSKIO	; lees sektor 0 van diskzijde 0
	POP DE
	POP BC
	JP C,L798E	; diskfout igv.SS -> eindig met 'Bad parameter'

; Hier begint het eigenlijke formatteren:

L7A09:	POP HL
	PUSH BC
	LD A,D
	PUSH AF
	LD DE,0
	CALL L76D6	; instelling kop-positie / diskzijde op 0
	POP AF
	POP BC
	CALL L7AAD	; vul einde buffer met info voor 'uitloop' v.e.track
	PUSH AF
L7A19:	RES 7,C
	CALL L7A4E	; vul buffer met track-inhoud zijde 0
	CALL L7BD8	; schrijf spoor op zijde 0
	JP C,L7999	; breek af in geval van fout
	BIT 0,C 	; SS ?
	JR Z,L7A36	; zo ja, schrijven diskzijde 1 overslaan
	CALL L7A4E	; vul buffer met track-inhoud zijde 1
	SET 7,C
	CALL L7A8E	; pas track-inhoud in buffer aan voor diskzijde 1
	CALL L7BD8	; schrijf track op diskzijde 1
	JP C,L7999	; breek af igv. diskfout
L7A36:	RES 5,D
	CALL L77D2	; drive-kop 1 stap opschuiven
	DJNZ L7A19	; verder met overblijvende tracks
	CALL L77FB	; zet kop terug naar spoor 0
	SET 7,C
	POP AF
	JP L7AC0	; ga verder met schrijven boot/FAT/dir.-sektoren

; Deze kleine routine dient voor het even instellen v.e. bepaalde diskzijde:

L7A46:	LD A,C
	RLCA
	AND %00000001
	LD (SSELREG),A
	RET

; Deze routine vult de formatteer-buffer met de volledige data voor 1 heel
; spoor op disk, alles bij elkaar zo'n kleine 7000 bytes:

L7A4E:	PUSH BC
	PUSH HL
	EX DE,HL
	LD HL,L7BAC	; tabelletje met info over invulling begin track
	CALL L7B79	; zet dit in buffer
	LD B,9		; aantal sektoren op 1 track/diskzijde
L7A59:	PUSH BC
	LD HL,L7BB7	; tabel met info over invulling v. 1 sektor-deel
	CALL L7B79	; aanvullen track-inhoud hiermee
	POP BC
	DJNZ L7A59	; track-inhoud aanvullen voor de 9 sektoren
	POP HL
	PUSH HL
	LD BC,138	; offset voor tracknrs. in sector-ID's
	ADD HL,BC
	LD DE,628	; totale grootte 1 sector
	LD B,9
	LD A,(TRACKREG)
L7A71:	LD (HL),A	; zet spoornr. in sector-ID
	ADD HL,DE
	DJNZ L7A71	; doe dit voor alle 9 sectoren
	LD BC,140	; offset voor sectornrs. zelf in sector-ID's
	POP HL
	PUSH HL
	ADD HL,BC
	PUSH HL
	POP IY
	LD HL,L7AA4	; tabelletje met sectornrs. (1-9)
	LD B,9
L7A83:	LD A,(HL)
	LD (IY+0),A	; zet sector-nr. in sector-ID
	ADD IY,DE
	INC HL
	DJNZ L7A83	; doe dit voor alle sectoren op het spoor
	POP HL
	POP BC
L7A8E:	PUSH BC
	PUSH HL
	LD DE,139	; offset voor diskzijde in sector-ID
	ADD HL,DE
	LD DE,628
	LD B,9
	LD A,C
	RLCA
	AND %00000001
L7A9D:	LD (HL),A	; zet diskzijde in sector-ID
	ADD HL,DE
	DJNZ L7A9D	; doe dit voor alle sectoren op het spoor
	POP HL
	POP BC
	RET

; Dit tabelletje staat voor de nummering van opeenvolgende sect. op 1 spoor:

L7AA4:	DB 1,2,3,4,5,6,7,8,9

; Hier wordt het einde van een track aangevuld met 'lege bytes' om het
; (niet gebruikte) laatste stuk van een spoor te vullen:

L7AAD:	PUSH HL
	PUSH BC
	LD DE,5774	; aantal bytes van begin-info + 9 * sektor-info
	ADD HL,DE
	LD D,H
	LD E,L
	INC DE
	LD (HL),#4E	; waarde gebruikt om tussenruimtes op track te vullen
	LD BC,1023
	LDIR		; vul track-buffer aan t/m 6798 bytes
	POP BC
	POP HL
	RET

; Hier wordt tenslotte een blanco inhoudsopgave op disk gezet:

L7AC0:	CALL L7B65	; aanmaken sector met 0-bytes in pag.3-sectorbuffer
	PUSH BC
	LD HL,L7E37	; adres bootsektor-info
	LD DE,(SECBUF)
	LD BC,ENDBOOT-BOOTSEKT
	LDIR		; zet bootsektor-info in sectorbuffer
L7AD0:	POP BC
	CALL L7B86	; pas bootsektor evt. aan voor DS disk
	PUSH BC
	LD B,1
	LD DE,0
	LD H,A
	PUSH HL
	LD HL,(SECBUF)
	SCF
	CALL DSKIO	; schrijf bootsektor naar disk
	POP HL
	POP BC
	RET C		; breek af igv. diskfout
	CALL L7B1C	; schrijf beide FATs naar disk
	RET C		; breek af igv. diskfout

; Tot slot worden hier nog een aantal lege directory-sectoren weggeschreven,
; in 2 blokken tbv. de verschillen tussen SS en DS disks:

	LD A,9
	SUB E
	LD B,A
L7AEE:	PUSH BC
	PUSH DE
	LD B,1
	LD A,H
	PUSH HL
	LD HL,(SECBUF)
	SCF
	CALL DSKIO	; wegschrijven 1 directory-sector
	POP HL
	POP DE
	INC DE
	POP BC
	RET C		; breek af igv. diskfout
	DJNZ L7AEE	; herhaal voor meerdere
	BIT 0,C 	; SS of DS ?
	RET Z
	LD B,5		; rest-aantal directory-sektoren
L7B07:	PUSH BC
	PUSH DE
	LD A,H
	PUSH HL
	LD HL,(SECBUF)
	LD B,1
	SCF
	CALL DSKIO	; wegschrijven 1 directory-sector
	POP HL
	POP DE
	INC DE
	POP BC
	RET C		; breek af igv. diskfout
	DJNZ L7B07	; wegschrijven rest lege directory
	RET		; klaar met formatteren


; Deze routine schrijft lege FATs naar een nieuwe diskette:

L7B1C:	LD B,2		; 2 FATs
	LD DE,1 	; 1e FAT-sektor
	CALL L7B65	; aanmaken 0-sektor
L7B24:	PUSH BC
	PUSH DE
	LD A,H
	PUSH HL
	LD HL,(SECBUF)

; Invulling eerste byte v.e.FAT:

	LD (HL),C	; media descriptor
	INC HL
	LD (HL),#FF
	INC HL
	LD (HL),#FF	; eerste cluster(s) niet gebruikt
	LD B,1
	DEC HL
	DEC HL
	SCF
	CALL DSKIO	; wegschrijven 1e sektor v.e. FAT
	POP HL
	CALL L7B65	; maak opnieuw 0-sektor aan
	POP DE
	INC DE
	POP BC
	RET C		; breek af in geval van disk-fout
	PUSH BC
	LD B,1
	BIT 0,C 	; SS of DS ?
	JR Z,L7B4A	; SS -> nog 2 FAT-sektoren te schrijven
	INC B		; DS -> nog 3
L7B4A:	PUSH BC
	LD B,1
	PUSH DE
	LD A,H
	PUSH HL
	LD HL,(SECBUF)
	SCF
	CALL DSKIO	; schrijf 1 volgende FAT sektor
	POP HL
	POP DE
	INC DE
	POP BC
	JR C,L7B63	; breek af igv.diskfout
	DJNZ L7B4A	; vervolg met schrijven rest lege FAT-sektoren
	POP BC
	DJNZ L7B24	; vervolg met schrijven andere FAT(s)
	RET

L7B63:	POP BC
	RET

; Deze routine zet een met nullen gevulde sector klaar in een pag.3-buffer:

L7B65:	PUSH BC
	PUSH DE
	PUSH HL
	LD HL,(SECBUF)
	LD D,H
	LD E,L
	INC DE
	LD BC,511
	LD (HL),0
	LDIR
	POP HL
	POP DE
	POP BC
	RET

; Deze routine dient voor het aanvullen van een stuk track-inhoud:

L7B79:	LD A,(HL)
	INC HL
	CP #FF		; afsluiter ?
	RET Z
	LD B,(HL)
	INC HL
L7B80:	LD (DE),A
	INC DE
	DJNZ L7B80
	JR L7B79

; Deze routine past een te schrijven bootsektor aan voor het geval dat het om
; een DS disk gaat:

L7B86:	BIT 0,C
	RET Z
	PUSH BC
	LD DE,L7B9C	; 'patch' voor DS bootsektor-info
	LD HL,(SECBUF)
	LD BC,13
	ADD HL,BC
	EX DE,HL
	LD BC,16
	LDIR
	POP BC
	RET

; Hier staat de bootsektor-info voor DS disks:

L7B9C:	DB 2		; sektoren/cluster
	DW 1		; gereserveerde sektoren
	DB 2		; aantal FATs
	DW 112		; aantal elementen in directory
	DW 1440 	; totaal aantal sectoren
	DB #F9		; media descriptor
	DW 3		; aantal sektoren in 1 FAT
	DW 9		; aantal sektoren per spoor
	DW 2		; aantal koppen (diskzijden)
	DB 0		; aantal verborgen sectoren

; Hier staat een deel van de invulling van een track bij het formatteren
; Formaat: byte-waarde, aantal keer (0=256),...,#FF

; Het begin van een track (122 bytes):

L7BAC:	DB #4E,80,#00,12,#F6,3
	DB #FC,1	; index mark
	DB #4E,26
	DB #FF		; afsluiter

; Het deel dat een sektor uitmaakt (totaal 628 bytes):

L7BB7:	DB #00,12,#F5,3
	DB #FE,1	; sector-ID adress mark
	DB #00,1	; hier wordt het track-nummer ingevuld
	DB #00,1	; hier wordt de diskzijde ingevuld
	DB #01,1	; hier wordt het sectornummer (1-9) ingevuld
	DB #02,1	; geeft sectorlengte van 512 bytes aan
	DB #F7,1	; zorgt voor schrijven 2-byte CRC
	DB #4E,24,#00,12,#F5,3
	DB #FB,1	; data adress mark
	DB #E5,0,#E5,0	; invulling sector
	DB #F7,1	; zorgt voor schrijven 2-byte CRC
	DB #4E,54	; ruimte tussen sectoren
	DB #FF

; Hier vindt het schrijven van 1 heel spoor naar disk plaats:

L7BD8:	CALL L7A46	; instelling side-selectie
	LD E,#0B	; bepaald 'zaag'-gedrag bij CRC-errors ed.
L7BDD:	PUSH BC
	PUSH DE
	PUSH HL
	CALL L77F4	; wacht tot FDC klaar is voor een opdracht
	CALL L780A	; wacht ca. 20 ms. (settling time)
	CALL DISINT
	DI		; geen interrupts tijdens formatteren v.h. spoor
	LD DE,L7C1D	; adres routine voor einde van formatteren spoor
	PUSH DE
	LD BC,INTREG
	LD DE,0 	; bepaald max. wachttijd op startsein FDC (=indexpuls)
	LD A,#F0	; FDC-commando 'write track'
	LD (CMDREG),A	; geef FDC opdracht voor beschrijven heel spoor
L7BF9:	LD A,(BC)	; toestand FDC interrupt/data request
	ADD A,A
	JR NC,L7C16	; data request -> geef eerste byte door aan FDC
	RET P		; interrupt request -> not ready/ write protect
	DEC E
	JR NZ,L7BF9	; blijf wachten op signaal FDC
	LD A,(BC)
	ADD A,A
	JR NC,L7C16
	RET P
	DEC D
	JR NZ,L7BF9
	POP BC
	LD A,16 	; foutcode: 'other error' (alleen bij formatteren)
	SCF
	POP HL
	POP DE
	POP BC
	RET
L7C11:	LD A,(BC)
	ADD A,A
	RET P
	JR C,L7C11	; wacht op signaal FDC om een volgende byte
L7C16:	LD A,(HL)	; haal byte uit formatteer-buffer
	LD (DATAREG),A	; geef door aan FDC
	INC HL
	JR L7C11	; wacht op volgende data request

; Hier eindigt het schrijven van 1 spoor:

L7C1D:	POP HL
	POP DE
	POP BC
	LD A,(STATREG)
	AND %11111100	; diskfout opgetreden ?
	JR Z,L7C42	; zo nee, ga verder met evt. teruglezen sectoren
	BIT 6,A 	; geeft FDC write protect aan ?
	JP NZ,L7545	; zo ja, stel vast werkelijk write prot. of not ready
	PUSH AF
	LD A,(TRACKREG)
	LD C,A
	CALL L77DB	; evt. opnieuw plaatsen kop op zelfde spoor
	POP AF
	DEC E		; aantal gedane pogingen
	JR NZ,L7BDD
	SCF
	LD E,A
	BIT 7,E 	; not ready-bit FDC gezet ?
	LD A,16 	; foutcode: 'other error'
	RET NZ
	JP L752F	; uitzoeken evt. andere fouten
L7C42:	LD A,(RAWFLG)	; verify-vlag
	OR A		; foutcontrole aan ?
	RET Z		; zo nee, klaar met schrijven v.h. spoor
	PUSH HL
	PUSH DE
	PUSH BC
	LD D,0
	LD A,C
	AND %00000001
	RRCA
	RRCA
	OR D
	LD D,A
	CALL L7A46	; instelling diskzijde
	OR D
	LD D,A
	LD B,9		; 9 sektoren teruglezen
	LD A,(TRACKREG)
	LD C,A
	LD A,1
	LD (SECTREG),A
	CALL L75D8	; lees net geschreven sectoren weer terug
	POP BC
	POP DE
	POP HL
	RET

; Hier wordt naar toe gesprongen wanneer een CALL-statement geen _SYSTEM of
; _FORMAT was, deze routine kan gebruikt worden om extra CALL-statements
; toe te voegen:

OEMSTAT:SCF		; geef aan dat statement-naam niet herkend is
	RET		; terug naar BASIC-interpreter

; Hier staat een gecombineerde tabel met relocate-informatie over de routines
; op 74D8-7565 en 77A1-781CH:

L7C6C:	DW L7D92-L7D33+L74DA-L74D8
	DW L7D92-L7D33+L7566-L74D8+L77F4-L77A1
	DW L7D92-L7D33+L74EE-L74D8
	DW L7D92-L7D33+L7504-L74D8
	DW L7D92-L7D33+L7520-L74D8
	DW L7D92-L7D33+L7566-L74D8+L77DB-L77A1
	DW L7D92-L7D33+L7515-L74D8
	DW L7D92-L7D33+L7566-L74D8+L77A1-L77A1
	DW L7D92-L7D33+L7518-L74D8
	DW L7D59-L7D33
	DW L7D92-L7D33+L7563-L74D8
	DW L7D6F-L7D33
	DW L7D92-L7D33+L7566-L74D8+L77A1-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77F4-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77D2-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77F4-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77DE-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77FB-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77E1-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77F4-L77A1
	DW L7D92-L7D33+L7566-L74D8+L7802-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77F4-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77FF-L77A1
	DW L7D92-L7D33+L7566-L74D8+L7815-L77A1
	DW L7D92-L7D33+L7566-L74D8+L7818-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77E1-L77A1
	DW 0

; En hier voor deze routines de adressen waarop een instructie staat die
; iets met een FDC-register doet, om deze voor pag.2 te kunnen aanpassen:

	DW L7D92-L7D33+L74F5-L74D8
	DW L7D92-L7D33+L74F2-L74D8
	DW L7D92-L7D33+L74FE-L74D8
	DW L7D92-L7D33+L7507-L74D8
	DW L7D92-L7D33+L7547-L74D8
	DW L7D92-L7D33+L754A-L74D8
	DW L7D92-L7D33+L7566-L74D8+L77A6-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77AA-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77B9-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77C6-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77CE-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77D7-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77E8-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77EF-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77F4-L77A1
	DW L7D92-L7D33+L7566-L74D8+L7807-L77A1
	DW L7D92-L7D33+L7566-L74D8+L77FB-L77A1
	DW L7D33-L7D33

; Hier staat een gecombineerde tabel tbv. het relocaten van routines op
; 761F-76A5, 77A1-781C en 7D33-7D91H:

L7CC6:	DW L7D92-L7D33+L761F-L761F
	DW L7D92-L7D33+L76A6-L761F+L77F4-L77A1
	DW L7D92-L7D33+L7633-L761F
	DW L7D92-L7D33+L766E-L761F
	DW L7D92-L7D33+L7686-L761F
	DW L7D92-L7D33+L76A6-L761F+L77DB-L77A1
	DW L7D92-L7D33+L767F-L761F
	DW L7D92-L7D33+L76A6-L761F+L77A1-L77A1
	DW L7D92-L7D33+L7682-L761F
	DW L7D59-L7D33
	DW L7D92-L7D33+L76A3-L761F
	DW L7D6F-L7D33
	DW L7D92-L7D33+L765B-L761F
	DW L7D92-L7D33+L76A3-L761F
	DW L7D92-L7D33+L76A6-L761F+L77A1-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77F4-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77D2-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77F4-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77DE-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77FB-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77E1-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77F4-L77A1
	DW L7D92-L7D33+L76A6-L761F+L7802-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77F4-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77FF-L77A1
	DW L7D92-L7D33+L76A6-L761F+L7815-L77A1
	DW L7D92-L7D33+L76A6-L761F+L7818-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77E1-L77A1
	DW 0

; Hier voor dezelfde routines een tabel met adressen waarop een FDC-register
; aangesproken wordt:

	DW L7D92-L7D33+L7637-L761F
	DW L7D92-L7D33+L763E-L761F
	DW L7D92-L7D33+L7663-L761F
	DW L7D92-L7D33+L7671-L761F
	DW L7D92-L7D33+L76A6-L761F+L77A6-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77AA-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77B9-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77C6-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77CE-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77D7-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77E8-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77EF-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77F4-L77A1
	DW L7D92-L7D33+L76A6-L761F+L7807-L77A1
	DW L7D92-L7D33+L76A6-L761F+L77FB-L77A1
	DW 0

; Deze routine verplaatst een paar hulproutines voor disk-I/O naar pag.0/1
; naar een sectorbuffer in pag.3:

L7D20:	LD DE,(SECBUF)
	LD HL,L7D33
	LD BC,L7D92-L7D33
	LDIR
	RET

; Langs deze routine worden naar pag.3 verplaatste routines voor het lezen
; v.e. sektor aangeroepen:

L7D2D:	PUSH HL
	LD HL,(SECBUF)	; beginadres verplaatste routines
	EX (SP),HL
	RET

; De volgende routine(s) worden bij disk-I/O van/naar pag.0 of 1 verplaatst
; naar de (FAT-)sektorbuffer op adres (F34DH):

L7D33:	PUSH HL
	PUSH DE
	PUSH BC
	CALL GETSLOT	; bepaal diskROM-slot
	LD HL,(SECBUF)	; beginadres v. deze routine
	LD DE,L7D82+1-L7D33
	ADD HL,DE
	LD (HL),A	; vul slot in in routine voor ordelijke terugkeer
	LD H,#80
	CALL ENASLT
	EI
	CALL ENAINT
	LD A,(RAMAD1)
	LD H,#40
	CALL ENASLT
	POP BC
	POP DE
	POP HL
	CALL DISINT
	DI
L7D59:	DEC HL
	LD A,H
	ADD A,2
	INC HL
	RLCA
	CCF		; waarsch. bepaling of doelgebied sector grens tussen
	JR NC,L7D6F	; pag.1 en 2 overlapt
	LD IY,(SECBUF)
	PUSH DE
	LD DE,L7D92-L7D33
	ADD IY,DE
	POP DE
	JP (IY) 	; spring direkt naar routine die hierachter komt
L7D6F:	PUSH HL
	PUSH DE
	PUSH BC
	PUSH AF
	LD A,(RAMAD2)
	LD H,#80
	CALL ENASLT
	CALL DOSON	; inschakelen main diskROM in pag.1
	EI
	CALL ENAINT
L7D82:	LD A,0		; hier wordt diskROM-slot ingevuld
	LD H,#40
	CALL ENASLT	; schakel diskROM zelf weer in pag.1
	EI
	CALL ENAINT
	POP AF
	POP BC
	POP DE
	POP HL
	RET

; Hier worden verscheidene routines, die samen dienen voor het schrijven van
; 1 of meer sectoren, verplaatst naar een sectorbuffer in pag.3:

L7D92:	PUSH HL
	PUSH DE
	PUSH BC
	CALL L7D20	; installatie hulproutines voor slotswitchen
	LD IY,L7C6C	; gecombineerde tabel met relocatie-adressen
	LD HL,L74D8	; routine voor schrijven v.e. sektor
	LD BC,L7566-L74D8
	LDIR		; zet direkt achter slotswitch-routines in sekt.buffer
	PUSH DE
	LD DE,L7D92-L7D33+L7563-L74D8
	JR L7DE3	; vervolg met installatie routine voor kop-bewegingen

; Hier worden een aantal hulproutines voor het verifieren van sectoren
; geinstalleerd:

L7DAA:	PUSH HL
	PUSH DE
	PUSH BC
	CALL L7DCD	; installeer routines als voor lezen sectoren
	LD HL,(SECBUF)	; startadres hulproutines = begin pag.3-sectorbuffer
	LD BC,L7D92-L7D33+L7666-L761F
	ADD HL,BC
	LD DE,L76CE	; begin routine(tje) voor vergelijken ipv. lezen
	EX DE,HL
	LD BC,L76D6-L76CE
	LDIR		; pas sektor-leesroutine aan om te verifieren
	LD HL,(SECBUF)
	LD BC,L7D92-L7D33+L763C+1-L761F
	ADD HL,BC
	LD (HL),0	; stel een JR buiten gebruik die dient voor
			; uitvoering verificatie rechtstreeks vanuit diskROM
	POP BC
	POP DE
	POP HL
	RET

L7DCD:	PUSH HL
	PUSH DE
	PUSH BC
	CALL L7D20	; kopieer hulproutines naar FAT-sektorbuffer
	LD IY,L7CC6	; gekombineerde relocatie-tabel voor div.routines
	LD HL,L761F
	LD BC,L76A6-L761F
	LDIR		; zet routine voor lezen sector hier achter aan
	PUSH DE
	LD DE,L7D92-L7D33+L76A3-L761F
L7DE3:	LD HL,(SECBUF)	; beginadres verplaatste routines
	ADD HL,DE
	LD (HL),#C3	; zet jump aan einde sektorlees-routine
	POP DE
	LD HL,L77A1
	LD BC,INIHRD-L77A1
	LDIR		; zet routine voor kop-bewegingen hier achter aan
L7DF2:	LD E,(IY+0)
	LD D,(IY+1)	; DE=offset tov. begin verplaatste routines
	LD A,D
	OR E		; einde met relocatie-adressen bereikt ?
	JR Z,L7E2D
	LD HL,(SECBUF)	; beginadres verplaatste routines
	ADD HL,DE	; beginadres v.e. aan te passen instructie
	INC HL		; overslaan 1e byte instructie
	PUSH HL
	LD E,(IY+2)
	LD D,(IY+3)	; offset mbt. adres genoemd in instructie
	LD HL,(SECBUF)
	ADD HL,DE	; bereken nieuw adres om in instructie in te vullen
	EX DE,HL
	POP HL
	LD (HL),E
	INC HL
	LD (HL),D	; vul nieuw adres in in instructie
	INC IY
	INC IY
	INC IY
	INC IY
	JR L7DF2

; Hier worden nog de adressen waarlangs de FDC aangesproken wordt, aangepast
; voor aanspreken via pag.2 ipv. pag.1:

L7E1B:	LD E,(IY+0)
	LD D,(IY+1)	; lees offset tov. begin verplaatste routines
	LD A,D
	OR E		; einde lijst bereikt ?
	JR Z,L7E33
	LD HL,(SECBUF)
	ADD HL,DE
	INC HL
	INC HL
	LD (HL),#BF	; pas high byte aanspreek-adres aan voor pag.2
L7E2D:	INC IY
	INC IY
	JR L7E1B
L7E33:	POP BC
	POP DE
	POP HL
	RET

; Hier staat de inhoud van de bootsektor, zoals die na het formatteren op
; disk gezet wordt (hier voor SS disks, wordt evt. voor DS aangepast):

L7E37:
	ORG #C000	; adres waar bootsektor heen wordt gekopieerd tijdens
			; opstart-procedure

BOOTSEKT:
	DB #EB,#FE,#90	; 8086 (!) jump-instructie
	DB "VG 8245 "   ; 'fabrikant-naam'
	DW 512		; bytes/sektor
	DB 2		; sektoren/cluster
	DW 1		; gereserveerde sektoren
	DB 2		; aantal FATs
	DW 112		; aantal elementen in directory
	DW 720		; totaal aantal sektoren
	DB #F8		; media descriptor
	DW 2		; aantal sektoren in 1 FAT
	DW 9		; aantal sektoren per spoor
	DW 1		; aantal koppen (diskzijden)
	DW 0		; aantal verborgen sectoren

; Hier staat de bootsektor-opstartroutine:

	RET NC		; niks doen indien 'zero page' niet geinitialiseerd is
	LD (LC058+1),DE ; routine voor inschakelen main diskrom in pag.1
	LD (ENDBOOT),A	; vlag mbt. uivoering AUTOEXEC.BAT
	LD (HL),#56
	INC HL
	LD (HL),#C0	; pointer naar foutafhandelingsroutine
LC02B:	LD SP,#F51F
	LD DE,LC079
	LD C,#0F
	CALL BDOS	; probeer "MSXDOS.SYS" file te openen
	INC A		; file gevonden ?
	JP Z,LC063	; zo nee, spring naar BASIC
	LD DE,#0100
	LD C,#1A
	CALL BDOS	; zet laad-adres
	LD HL,1
	LD (LC079+14),HL ; zet record-lengte
	LD HL,#3F00	; max.grootte MSXDOS.SYS (rest pag.0)
	LD DE,LC079
	LD C,#27
	CALL BDOS	; lees MSXDOS.SYS in op 0100H
	JP #0100	; spring naar begin hiervan
	DW LC058	; pointer naar begin foutroutine
LC058:	CALL 0		; wordt call voor inschakelen main diskROM in pag.1
	LD A,C
	AND %11111110
	CP %00000010	; diskfout is 'not ready' (!?!) ?
	JP NZ,LC06A	; zo nee, geef 'Boot error' en probeer opnieuw
LC063:	LD A,(ENDBOOT)
	AND A		; boot-poging na reset of later ?
	JP Z,#4022	; bij reset -> BASIC opstarten
LC06A:	LD DE,LC09E	; in andere gevallen:
	LD C,9
	CALL BDOS	; melding 'boot error ...'
	LD C,7
	CALL BDOS	; wacht op toetsdruk
	JR LC02B	; herstel stack en probeer opnieuw

; File Control Block voor deze bootsektor-routine:

LC079:	DB 0
	DB "MSXDOS  SYS"
	DS 25,0 	; totaal 37 bytes

LC09E:	DB "Boot error",13,10
	DB "Press any key for retry",13,10
	DB "$"

ENDBOOT:

	ORG L7E37+ENDBOOT-BOOTSEKT

; De volgende ruimte wordt verder niet meer gebruikt:

	DB 0
	DB "v 1.05",26

	DS #8000-$,#FF	; opvullen met 1-bitjes


	END



To fit this code into a complete diskROM, take these steps:

-Assemble this source (I used v2.04 of the popular GEN80 assembler)
-Make a note of the labels mentioned below
 (or make a printout of the .SYM-file)
-Use a MSX debugger to load the original diskROM in RAM, starting at
 address 4000h
-Load the assembled code 'over' it, starting at address 7405h
-Adjust the following addresses, using the labels below:


4010: JP DSKIO
4013: JP DSKCHG
4016: JP GETDPB
4019: JP CHOICE
401C: JP DSKFMT
401F: JP MTOFF
576F: CALL INIHRD
5850: CALL DRIVES
5883: LD HL,DEFDPB-1
588F: CALL INIENV
65AE: JP OEMSTAT


These addresses might also be adjusted, if desired:

57A9: LD HL,F380h+MYSIZE
581D: LD HL,MYSIZE	; size of diskROM work area (originally 8 bytes)
582E: LD DE,SECLEN	; maximum sectorsize used by this diskROM


-Save the resulting code (4000-7FFFh, 16 KB.)


